BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'DELETED_APPROVERS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM DELETED_APPROVERS_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'DELETED_APPROVERS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW DELETED_APPROVERS_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ROLE_POLICY_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ROLE_POLICY_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ROLE_POLICY_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ROLE_POLICY_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ROLE_POLICY_ALL_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ROLE_POLICY_ALL_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ROLE_POLICY_ALL_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ROLE_POLICY_ALL_V';
  END LOOP;
END;
/

CREATE VIEW deleted_approvers_v AS
SELECT DISTINCT
  approvals_info.id,
  approvals_info.current_approval_step,
  approvals_info.request_policy_id,
  request_pol.name AS request_policy_name,
  approvals_info.approval_policy_id,
  approval_pol.name AS approval_policy_name,
  approvals_info.sod_unique_id,
  approvals_info.sod_policy,
  approvals_info.sod_policy_owner,
  approvals_info.psodv_approval_start_time,
  CASE
    WHEN approvals_info.psodv_approval_start_time IS NOT NULL THEN approvals_info.sod_policy_owner
    ELSE TO_NCHAR(approvers.approver)
  END AS deleted_approver_unique_id,
  CASE
    WHEN (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL) THEN psod_su.display_name
    WHEN (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL) THEN psod_sg.name
    WHEN (approvals_info.psodv_approval_start_time IS NULL AND su.id IS NOT NULL) THEN su.display_name
    ELSE sg.name
  END AS deleted_approver,
  CASE
    WHEN (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL)
      OR (approvals_info.psodv_approval_start_time IS NULL AND su.id IS NOT NULL) THEN 1
    WHEN (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL)
      OR (approvals_info.psodv_approval_start_time IS NULL AND sg.id IS NOT NULL) THEN 0
    ELSE NULL
  END AS is_user,
  CASE
    WHEN (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL) THEN psod_su.timestamp
    WHEN (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL) THEN psod_sg.timestamp
    WHEN (approvals_info.psodv_approval_start_time IS NULL AND su.id IS NOT NULL) THEN su.timestamp
    ELSE sg.timestamp
  END AS timestamp,
  approvals_info.unique_item_id,
  approvals_info.item_request_reason,
  approvals_info.item_type,
  approvals_info.add_item,
  approvals_info.unique_requester_id,
  approvals_info.request_status,
  approvals_info.unique_recipient_id,
  approvals_info.item_approval_status,
  approvals_info.creation_time,
  approvals_info.creation_time + interval '1' day * aas.timeout_period AS approval_expiration,
  aas.timeout_action,
  approvals_info.approval_start_time,
  approvals_info.last_status_update_time,
  approvals_info.status_changed_by
FROM (
  SELECT DISTINCT
    jt.step_number AS step,
    jt.approvers AS approvers,
    rr.id,
    rr.current_approval_step,
    rr.unique_requester_id,
    rr.request_status,
    rr.approval_policy_id,
    rr.creation_time,
    rr.approval_start_time,
    rr.last_update_time,
    sodp.unique_policy_id AS sod_unique_id,
    sodp.name AS sod_policy,
    po.unique_owner_id AS sod_policy_owner,
    rr.psodv_approval_start_time,
    rri.id AS request_item_id,
    rri.request_policy_id,
    rri.unique_item_id,
    rri.reason AS item_request_reason,  
    rri.item_type,
    rri.add_item,
    rri.unique_recipient_id,
    rri.item_approval_status,
    rri.last_status_update_time,
    rri.status_changed_by
  FROM resource_request_item rri
  JOIN resource_request rr ON rr.id = rri.resource_request_id AND rr.request_status IN ('APPROVING','APPROVING_PSODV')
  JOIN JSON_TABLE(rr.approval_strategy_json, '$.approvals[*]' columns(step_number FOR ORDINALITY, approvers VARCHAR2(2000) FORMAT JSON WITH ARRAY WRAPPER PATH '$.approvers[*]')) AS jt ON 1 = 1
  LEFT JOIN psodv_approval_task_item pati ON pati.resource_request_item_id = rri.id
  LEFT JOIN psodv_approval_task pat ON pat.id = pati.psodv_approval_task_id
  LEFT JOIN sod_policy sodp ON sodp.id = pat.policy_id
  LEFT JOIN policy_owner po ON po.sod_policy = sodp.id
  WHERE 
    rri.item_approval_status IN ('NO_DECISION', 'TENTATIVE_APPROVED', 'TENTATIVE_DENIED','WAITING_PSODV_APPROVAL')
) approvals_info
JOIN JSON_TABLE(approvals_info.approvers, '$[*]' columns(approver varchar2(255) path '$[0]')) AS approvers ON 1 = 1
LEFT JOIN (SELECT 
    suser.id,
    suser.unique_user_id,
    suser.display_name,
    suser.effective,
    suser.deleted,
    COALESCE(suser.last_modified,
      (SELECT min(sversion.timestamp) AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)
    ) AS timestamp
  FROM suser) su ON su.unique_user_id = approvers.approver AND su.effective = 1
LEFT JOIN (SELECT 
    suser.id,
    suser.unique_user_id,
    suser.display_name,
    suser.effective,
    suser.deleted,
    COALESCE(suser.last_modified,
      (SELECT min(sversion.timestamp) AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)
    ) AS timestamp
  FROM suser) psod_su ON psod_su.unique_user_id = approvals_info.sod_policy_owner AND psod_su.effective = 1
LEFT JOIN (SELECT 
    sgroup.id,
    sgroup.unique_group_id,
    sgroup.name,
    sgroup.effective,
    sgroup.deleted,
    COALESCE(sgroup.last_modified,
      (SELECT min(sversion.timestamp) AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sgroup.sversion)
    ) AS timestamp
  FROM sgroup) sg ON sg.unique_group_id = approvers.approver AND sg.effective = 1
LEFT JOIN (SELECT 
    sgroup.id,
    sgroup.unique_group_id,
    sgroup.name,
    sgroup.effective,
    sgroup.deleted,
    COALESCE(sgroup.last_modified,
      (SELECT min(sversion.timestamp) AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sgroup.sversion)
    ) AS timestamp
  FROM sgroup) psod_sg ON psod_sg.unique_group_id = approvals_info.sod_policy_owner AND psod_sg.effective = 1
LEFT JOIN access_req_approval_policy approval_pol ON approval_pol.id = approvals_info.approval_policy_id
LEFT JOIN access_approval_step aas ON aas.policy_id = approvals_info.approval_policy_id AND aas.step_sequence = approvals_info.current_approval_step                                                         
LEFT JOIN access_req_policy request_pol ON request_pol.id = approvals_info.request_policy_id
WHERE (approvals_info.step = approvals_info.current_approval_step OR approvals_info.current_approval_step IS NULL) AND
  ((approvals_info.psodv_approval_start_time IS NULL AND su.id IS NOT NULL AND su.deleted = 1) OR
   (approvals_info.psodv_approval_start_time IS NULL AND sg.id IS NOT NULL AND sg.deleted = 1) OR
   (approvals_info.psodv_approval_start_time IS NULL AND su.id IS NULL AND sg.id IS NULL) OR
   (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL AND psod_su.deleted = 1) OR
   (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL AND psod_sg.deleted = 1) OR
   (approvals_info.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NULL AND psod_sg.id IS NULL));

CREATE VIEW role_policy_v AS

SELECT role_policy.id,
    role_policy.unique_policy_id,
    role_policy.deleted,
    role_policy.state,
    role_policy.description,
    role_policy.name,
    role_policy.risk,
    role_policy.created_by,
    role_policy.changed_by,
    role_policy.update_time,
    role_policy.creation_time,
    role_policy.detection_status,
    role_policy.risk_mode,
    role_policy.cost
   FROM role_policy
  WHERE role_policy.deleted = '0';

  CREATE VIEW role_policy_all_v AS 
  SELECT role_policy.id,
    role_policy.unique_policy_id,
    role_policy.deleted,
    role_policy.state,
    role_policy.description,
    role_policy.name,
    role_policy.risk,
    role_policy.created_by,
    role_policy.changed_by,
    role_policy.update_time,
    role_policy.creation_time,
    role_policy.detection_status,
    role_policy.risk_mode,
    role_policy.cost
  FROM role_policy;

GRANT SELECT ON igops.deleted_approvers_v TO ig_report_role;
CREATE PUBLIC SYNONYM deleted_approvers_v FOR igops.deleted_approvers_v;

GRANT SELECT ON igops.role_policy_v TO ig_report_role;
CREATE PUBLIC SYNONYM role_policy_v FOR igops.role_policy_v;

GRANT SELECT ON igops.role_policy_all_v TO ig_report_role;
CREATE PUBLIC SYNONYM role_policy_all_v FOR igops.role_policy_all_v;